/*****************************************************************************/
/*!
 * \file notifylist.h
 * 
 * Author: Clark Barrett
 * 
 * Created: Mon Jan 20 13:52:19 2003
 *
 * <hr>
 * Copyright (C) 2003 by the Board of Trustees of Leland Stanford
 * Junior University and by New York University. 
 *
 * License to use, copy, modify, sell and/or distribute this software
 * and its documentation for any purpose is hereby granted without
 * royalty, subject to the terms and conditions defined in the \ref
 * LICENSE file provided with this distribution.  In particular:
 *
 * - The above copyright notice and this permission notice must appear
 * in all copies of the software and related documentation.
 *
 * - THE SOFTWARE IS PROVIDED "AS-IS", WITHOUT ANY WARRANTIES,
 * EXPRESSED OR IMPLIED.  USE IT AT YOUR OWN RISK.
 * 
 * <hr>
 * 
 */
/*****************************************************************************/

#ifndef _cvcl__include__notifylist_h_ 
#define _cvcl__include__notifylist_h_  

#include "context.h"
#include "expr.h"

namespace CVCL {

class NotifyList: public ContextObj {
//   CDList<Theory*> d_tlist;
//   CDList<Expr> d_elist;
  std::vector<Theory*> d_tlist;
  std::vector<Expr> d_elist;
  //! Private copy constructor
  NotifyList(const NotifyList& nl)
    : ContextObj(nl), d_tlist(nl.d_tlist), d_elist(nl.d_elist) { }
  //! Disable assignment
  NotifyList& operator=(const NotifyList& nl) { return *((NotifyList*)NULL); }
  //! Make a copy of itself for push()
  ContextObj* makeCopy(void) { return new NotifyList(*this); }
  //! Restore itself on pop()
  void restoreData(ContextObj* data) {
    d_tlist = ((NotifyList*)data)->d_tlist;
    d_elist = ((NotifyList*)data)->d_elist;
  }
  //! Erase itself
  void setNull(void) { d_tlist.clear(); d_elist.clear(); }

public:
  NotifyList(Context* c) : ContextObj(c) {
    IF_DEBUG(setName("NotifyList"));
  }
  unsigned size() const { return d_tlist.size(); }
  void add(Theory* t, const Expr& e) {
    makeCurrent();
    d_tlist.push_back(t);
    d_elist.push_back(e);
  }
  void remove(Theory* t, const Expr& e) {
    makeCurrent();
    // Search for <t,e> pair index
    unsigned i=0, iend=size();
    for(; i<iend && d_tlist[i]!=t && d_elist[i]!=e; ++i);
    if(i!=iend) { // Excise the index from existence
      d_tlist[i] = d_tlist.back(); d_tlist.pop_back();
      d_elist[i] = d_elist.back(); d_elist.pop_back();
    }
  }
  Theory* getTheory(int i) const { return d_tlist[i]; }
  Expr getExpr(int i) const { return d_elist[i]; }
};

}

#endif
